"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const path = require("path");
exports.isComponentDecorator = (node) => {
    if (ts.isDecorator(node)) {
        const callExpression = node.getChildren().find(ts.isCallExpression);
        if (callExpression) {
            const identifier = callExpression.getChildren().find(ts.isIdentifier);
            return identifier && identifier.getText() === 'Component';
            /* TODO: text comparison can break when
            * `import { Component as foo } from '@angular/core'` or
            * `import * as ng from '@angular/core'`
            * @link https://github.com/angular/devkit/blob/master/packages/schematics/angular/utility/ast-utils.ts#L127-L128
            */
        }
    }
    return false;
};
exports.isPropertyAssignmentFor = (node, name) => ts.isPropertyAssignment(node) && node.name.getText() === name;
exports.isTemplateUrl = (node) => exports.isPropertyAssignmentFor(node, 'templateUrl');
exports.isStyleUrls = (node) => exports.isPropertyAssignmentFor(node, 'styleUrls');
exports.componentTransformer = ({ templateProcessor, stylesheetProcessor, sourceFileWriter }) => (context) => (sourceFile) => {
    // skip source files from 'node_modules' directory (third-party source)
    if (sourceFile.fileName.includes('node_modules')) {
        return sourceFile;
    }
    const sourceFilePath = sourceFile.fileName;
    const visitComponents = (node) => {
        if (exports.isTemplateUrl(node)) {
            // XX: strip quotes (' or ") from path
            const templatePath = node.initializer.getText().substring(1, node.initializer.getText().length - 1);
            const templateFilePath = path.resolve(path.dirname(sourceFilePath), templatePath);
            const template = templateProcessor(sourceFilePath, templatePath, templateFilePath);
            if (typeof template === 'string') {
                const synthesizedNode = ts.updatePropertyAssignment(node, ts.createIdentifier('template'), ts.createLiteral(template));
                if (sourceFileWriter) {
                    const synthesizedSourceText = 'template: `'.concat(template).concat('`');
                    sourceFileWriter(sourceFile, node, synthesizedSourceText);
                }
                return synthesizedNode;
            }
            else {
                return node;
            }
        }
        else if (exports.isStyleUrls(node)) {
            // handle array arguments for styleUrls
            const styleUrls = node.initializer.getChildren()
                .filter((node) => node.kind === ts.SyntaxKind.SyntaxList)
                .map((node) => node.getChildren().map(n => n.getText()))
                .reduce((prev, current) => prev.concat(...current), [])
                .filter(text => text !== ',')
                .map((url) => url.substring(1, url.length - 1));
            const stylesheets = styleUrls.map((url) => {
                const styleFilePath = path.resolve(path.dirname(sourceFilePath), url);
                const content = stylesheetProcessor(sourceFilePath, url, styleFilePath);
                return typeof content === 'string' ? content : url;
            });
            const hasChanged = stylesheets.every((value, index) => {
                return styleUrls[index] && styleUrls[index] !== value;
            });
            if (hasChanged) {
                const synthesizedNode = ts.updatePropertyAssignment(node, ts.createIdentifier('styles'), ts.createArrayLiteral(stylesheets.map((value) => ts.createLiteral(value))));
                if (sourceFileWriter) {
                    const synthesizedSourceText = 'styles: ['
                        .concat(stylesheets.map((value) => `\`${value}\``).join(', '))
                        .concat(']');
                    sourceFileWriter(sourceFile, node, synthesizedSourceText);
                }
                return synthesizedNode;
            }
            else {
                return node;
            }
        }
        return ts.visitEachChild(node, visitComponents, context);
    };
    const visitDecorators = (node) => exports.isComponentDecorator(node)
        ? ts.visitEachChild(node, visitComponents, context)
        : ts.visitEachChild(node, visitDecorators, context);
    return ts.visitNode(sourceFile, visitDecorators);
};
//# sourceMappingURL=ts-transformers.js.map